@@ -111,6 +111,8 @@ typedef struct r_asm_t {
 	char *features;
 	int invhex; // invalid instructions displayed in hex
 	int pcalign;
+	int dataalign;
+	SdbHash *flags;
 } RAsm;
 
 typedef int (*RAsmModifyCallback)(RAsm *a, ut8 *buf, int field, ut64 val);
