@@ -12,7 +12,6 @@ extern "C" {
 #include <sdb.h>
 
 #include <stdio.h>
-#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -26,8 +25,10 @@ extern "C" {
 #include <windows.h>
 #include <wincon.h>
 #endif
+#include <unistd.h>
 
 /* constants */
+#define ENUM_FOR_PAL 0
 #define CONS_MAX_USER 102400
 #define CONS_BUFSZ 0x4f00
 #define STR_IS_NULL(x) (!x || !x[0])
@@ -72,9 +73,76 @@ typedef struct r_cons_grep_t {
 	int neg;
 	int begin;
 	int end;
+	int icase;
 } RConsGrep;
 
+#if ENUM_FOR_PAL
+enum {
+	R_CONS_PAL_0x00 = 0,
+	R_CONS_PAL_0x7f,
+	R_CONS_PAL_0xff,
+	R_CONS_PAL_args,
+	R_CONS_PAL_bin,
+	R_CONS_PAL_btext,
+	R_CONS_PAL_CALL,
+	R_CONS_PAL_CJMP,
+	R_CONS_PAL_CMP,
+	R_CONS_PAL_COMMENT,
+	R_CONS_PAL_CREG,
+	R_CONS_PAL_FLAG,
+	R_CONS_PAL_FLINE,
+	R_CONS_PAL_FLOC,
+	R_CONS_PAL_FLOW,
+	R_CONS_PAL_FLOW2,
+	R_CONS_PAL_fname,
+	R_CONS_PAL_help,
+	R_CONS_PAL_input,
+	R_CONS_PAL_INVALID,
+	R_CONS_PAL_JMP,
+	R_CONS_PAL_LABEL,
+	R_CONS_PAL_MATH,
+	R_CONS_PAL_MOV,
+	R_CONS_PAL_NOP,
+	R_CONS_PAL_NUM,
+	R_CONS_PAL_OFFSET,
+	R_CONS_PAL_OTHER,
+	R_CONS_PAL_POP,
+	R_CONS_PAL_PROMPT,
+	R_CONS_PAL_PUSH,
+	R_CONS_PAL_CRYPTO,
+	R_CONS_PAL_REG,
+	R_CONS_PAL_RESET,
+	R_CONS_PAL_RET,
+	R_CONS_PAL_SWI,
+	R_CONS_PAL_TRAP,
+	R_CONS_PAL_AI_READ,
+	R_CONS_PAL_AI_WRITE,
+	R_CONS_PAL_AI_EXEC,
+	R_CONS_PAL_AI_SEQ,
+	R_CONS_PAL_AI_ASCII,
+	R_CONS_PAL_GUI_CFLOW,
+	R_CONS_PAL_GUI_DATAOFFSET,
+	R_CONS_PAL_GUI_BACKGROUND,
+	R_CONS_PAL_GUI_ALT_BACKGROUND,
+	R_CONS_PAL_GUI_BORDER,
+	R_CONS_PAL_GRAPH_BOX,
+	R_CONS_PAL_GRAPH_BOX2,
+	R_CONS_PAL_GRAPH_BOX3,
+	R_CONS_PAL_GRAPH_BOX4,
+	R_CONS_PAL_GRAPH_TRUE,
+	R_CONS_PAL_GRAPH_FALSE,
+	R_CONS_PAL_GRAPH_TRUFAE,
+	R_CONS_PAL_GRAPH_TRACED,
+	R_CONS_PAL_GRAPH_CURRENT,
+	R_CONS_PAL_LAST
+}
+#endif
+
 typedef struct r_cons_palette_t {
+// TODO: this must be an array of char *
+#if ENUM_FOR_PAL
+	char *color[R_CONS_PAL_LAST];
+#else
 	char *b0x00;
 	char *b0x7f;
 	char *b0xff;
@@ -133,11 +201,17 @@ typedef struct r_cons_palette_t {
 	char *graph_trufae;
 	char *graph_traced;
 	char *graph_current;
-
+#endif
 #define R_CONS_PALETTE_LIST_SIZE 8
 	char *list[R_CONS_PALETTE_LIST_SIZE];
+	char **rainbow; // rainbow
+	int rainbow_sz; // size of rainbow
 } RConsPalette;
 
+R_API const char *r_cons_rainbow_get(int idx, int last, bool bg);
+R_API void r_cons_rainbow_free();
+R_API void r_cons_rainbow_new(int sz);
+
 typedef void (*RConsEvent)(void *);
 
 #define CONS_MAX_ATTR_SZ 15
@@ -314,6 +388,7 @@ typedef struct r_cons_t {
 #define Color_BGMAGENTA  "\x1b[45m"
 #define Color_YELLOW     "\x1b[33m"
 #define Color_BGYELLOW   "\x1b[43m"
+#define Color_BGBYELLOW  "\x1b[1;43m"
 #define Color_CYAN       "\x1b[36m"
 #define Color_BGCYAN     "\x1b[46m"
 #define Color_BLUE       "\x1b[34m"
@@ -494,6 +569,7 @@ R_API int r_cons_eof(void);
 
 R_API int r_cons_palette_init(const unsigned char *pal);
 R_API int r_cons_pal_set (const char *key, const char *val);
+R_API void r_cons_pal_update_event();
 R_API void r_cons_pal_free(void);
 R_API void r_cons_pal_init(const char *foo);
 R_API char *r_cons_pal_parse(const char *str);
