@@ -4,14 +4,15 @@
 #include <r_types.h>
 #include <r_anal.h>
 #include <r_cons.h>
+#include <r_hash.h>
 #include <r_util.h>
 #include <r_reg.h>
+#include <r_egg.h>
 #include <r_bp.h>
-#include <r_db.h>
 #include <r_io.h>
 #include <r_syscall.h>
-#include "list.h"
 
+#include <r_config.h>
 #include "r_bind.h"
 #ifdef __cplusplus
 extern "C" {
@@ -44,23 +45,71 @@ R_LIB_VERSION_HEADER(r_debug);
 #define PTRACE_SYSCALL PT_STEP
 #endif
 
-enum {
+#define SNAP_PAGE_SIZE 4096
+
+/*
+ * states that a process can be in
+ */
+typedef enum {
 	R_DBG_PROC_STOP = 's',
 	R_DBG_PROC_RUN = 'r',
 	R_DBG_PROC_SLEEP = 'S',
 	R_DBG_PROC_ZOMBIE = 'z',
 	R_DBG_PROC_DEAD = 'd',
 	R_DBG_PROC_RAISED = 'R' // has produced a signal, breakpoint, etc..
-};
+} RDebugPidState;
 
 
 // signal handling must support application and debugger level options
-enum {
-	R_DBG_SIGNAL_IGNORE=0, // ignore signal handler
-	R_DBG_SIGNAL_CONT=1, // pass signal to chlidren and continue execution
-	R_DBG_SIGNAL_SKIP=2, //
+typedef enum {
+	R_DBG_SIGNAL_IGNORE = 0, // ignore signal handler
+	R_DBG_SIGNAL_CONT = 1, // pass signal to chlidren and continue execution
+	R_DBG_SIGNAL_SKIP = 2, //
 	//..
-};
+} RDebugSignalMode;
+
+
+/*
+ * when a user wants to resume from a breakpoint, we need to know how they want
+ * to proceed. these values indicate their intention.
+ */
+typedef enum {
+	R_DBG_RECOIL_NONE = 0,
+	R_DBG_RECOIL_STEP,
+	R_DBG_RECOIL_CONTINUE
+} RDebugRecoilMode;
+
+/*
+ * List of reasons that an inferior might have stopped
+ */
+typedef enum {
+	R_DEBUG_REASON_DEAD = -1,
+	R_DEBUG_REASON_NONE = 0,
+	R_DEBUG_REASON_SIGNAL,
+	R_DEBUG_REASON_SEGFAULT,
+	R_DEBUG_REASON_BREAKPOINT,
+	R_DEBUG_REASON_TRACEPOINT,
+	R_DEBUG_REASON_COND,
+	R_DEBUG_REASON_READERR,
+	R_DEBUG_REASON_STEP,
+	R_DEBUG_REASON_ABORT,
+	R_DEBUG_REASON_WRITERR,
+	R_DEBUG_REASON_DIVBYZERO,
+	R_DEBUG_REASON_ILLEGAL,
+	R_DEBUG_REASON_UNKNOWN,
+	R_DEBUG_REASON_ERROR,
+	R_DEBUG_REASON_NEW_PID,
+	R_DEBUG_REASON_NEW_TID,
+	R_DEBUG_REASON_NEW_LIB,
+	R_DEBUG_REASON_EXIT_PID,
+	R_DEBUG_REASON_EXIT_TID,
+	R_DEBUG_REASON_EXIT_LIB,
+	R_DEBUG_REASON_TRAP,
+	R_DEBUG_REASON_SWI,
+	R_DEBUG_REASON_INT,
+	R_DEBUG_REASON_FPU,
+} RDebugReasonType;
+
 
 /* TODO: move to r_anal */
 typedef struct r_debug_frame_t {
@@ -70,11 +119,12 @@ typedef struct r_debug_frame_t {
 	ut64 bp;
 } RDebugFrame;
 
+
 typedef struct r_debug_reason_t {
-        int type;
+	int /*RDebugReasonType*/ type;
 	int tid;
 	int signum;
-	RBreakpointItem *bpi;
+	ut64 bp_addr;
 	ut64 timestamp;
 	ut64 addr;
 	ut64 ptr;
@@ -85,9 +135,11 @@ typedef struct r_debug_map_t {
 	ut64 addr;
 	ut64 addr_end;
 	ut64 size;
+	ut64 offset;
 	char *file;
 	int perm;
 	int user;
+	bool shared;
 } RDebugMap;
 
 typedef struct r_debug_signal_t {
@@ -104,16 +156,45 @@ typedef struct r_debug_desc_t {
 	ut64 off;
 } RDebugDesc;
 
+struct r_debug_snap_diff_t;
+typedef struct r_page_data_t {
+	struct r_debug_snap_diff_t *diff; // Pointing SnapDiff that has this pagedata.
+	int page_off;
+	ut8 *data;
+	ut8 hash[128];
+} RPageData;
+
+struct r_debug_snap_t;
+typedef struct r_debug_snap_diff_t {
+	struct r_debug_snap_t *base;
+	RList *pages; // <RPageData*>
+	RPageData **last_changes; // Last diff entries of each pages
+} RDebugSnapDiff;
+
 typedef struct r_debug_snap_t {
 	ut64 addr;
 	ut64 addr_end;
 	ut8 *data;
 	ut32 size;
+	ut32 page_num;
 	ut64 timestamp;
-	ut32 crc;
+	RHash *hash_ctx;
+	ut8 **hashes; // Hash of each pages
+	RList *history; // <RDebugSnapDiff*>
 	char *comment;
 } RDebugSnap;
 
+typedef struct r_debug_key {
+	ut64 addr;
+	int id;
+} RDebugKey;
+
+typedef struct r_debug_session_t {
+	RDebugKey key;
+	RListIter *reg[R_REG_TYPE_LAST];
+	RList *memlist; // <RDebugSnapDiff*>
+} RDebugSession;
+
 typedef struct r_debug_trace_t {
 	RList *traces;
 	int count;
@@ -139,45 +220,65 @@ typedef struct r_debug_tracepoint_t {
 typedef struct r_debug_t {
 	char *arch;
 	int bits; /// XXX: MUST SET ///
-	int pid;    /* selected process id */
-	int tid;    /* selected thread id */
+	int hitinfo;
+
+	int main_pid;
+	int pid; /* selected process id */
+	int tid; /* selected thread id */
+	int forked_pid; /* last pid created by fork */
+	RList *threads; /* NOTE: list contents are platform-specific */
+
+	/* dbg.* config options (see e?dbg)
+	 * NOTE: some settings are checked inline instead of tracked here.
+	 */
+	int bpsize; /* size of a breakpoint */
+	char *btalgo; /* select backtrace algorithm */
+	int btdepth; /* backtrace depth */
+	int regcols; /* display columns */
 	int swstep; /* steps with software traps */
-	int steps;  /* counter of steps done */
-	int newstate;
+	int stop_all_threads; /* stop all threads at any stop */
+	int trace_forks; /* stop on new children */
+	int trace_execs; /* stop on new execs */
+	int trace_aftersyscall; /* stop after the syscall (before if disabled) */
+	int trace_clone; /* stop on new threads */
+	int follow_child; /* On fork, trace the child */
+	char *glob_libs; /* stop on lib load */
+	char *glob_unlibs; /* stop on lib unload */
+	bool consbreak; /* SIGINT handle for attached processes */
+
+	/* tracking debugger state */
+	int steps; /* counter of steps done */
 	RDebugReason reason; /* stop reason */
+	RDebugRecoilMode recoil_mode; /* what did the user want to do? */
+
+	/* tracing vars */
 	RDebugTrace *trace;
-	int stop_all_threads;
+	Sdb *tracenodes;
+	RTree *tree;
+
 	RReg *reg;
 	const char *creg; // current register value
 	RBreakpoint *bp;
-	int bpsize;
-	int btdepth;
-	void *user;
+	void *user; // XXX(jjd): unused?? meant for caller's use??
+
 	/* io */
 	PrintfCallback cb_printf;
-	struct r_debug_plugin_t *h;
-	struct list_head plugins;
-	RAnal *anal;
 	RIOBind iob;
+
+	struct r_debug_plugin_t *h;
+	RList *plugins;
+
+	RAnal *anal;
 	RList *maps; // <RDebugMap>
 	RList *maps_user; // <RDebugMap>
 	RList *snaps; // <RDebugSnap>
-	RTree *tree;
-	Sdb *tracenodes;
+	RList *sessions; // <RDebugSession>
 	Sdb *sgnls;
 	RCoreBind corebind;
-	int trace_forks;
-	int trace_execs;
-	int trace_clone;
 	// internal use only
 	int _mode;
-	RList *threads; // XXX This is platform-specific !!!
-	/* select backtrace algorithm */
-	char *btalgo;
 	RNum *num;
-	int regcols;
-	char *glob_libs;
-	char *glob_unlibs;
+	REgg *egg;
 } RDebug;
 
 typedef struct r_debug_desc_plugin_t {
@@ -202,18 +303,20 @@ typedef struct r_debug_info_t {
 	int status; // zombie, running, sleeping, ...
 	int signum;
 	void * lib;
+	void * thread;
+	char *kernel_stack;
 	// retrieve mem/fd/core limits?
 	// list of threads ? hasthreads? counter?
 	// environment?
-	// /proc/pid/stack ???
 	// /proc/pid/syscall ???
-	//
 } RDebugInfo;
 
 /* TODO: pass dbg and user data pointer everywhere */
 typedef struct r_debug_plugin_t {
 	const char *name;
 	const char *license;
+	const char *author;
+	const char *version;
 	//const char **archs; // MUST BE DEPREACTED!!!!
 	ut32 bits;
 	const char *arch;
@@ -226,16 +329,17 @@ typedef struct r_debug_plugin_t {
 	int (*detach)(RDebug *dbg, int pid);
 	int (*select)(int pid, int tid);
 	RList *(*threads)(RDebug *dbg, int pid);
-	RList *(*pids)(int pid);
-	RList *(*tids)(int pid);
-	RFList (*backtrace)(int count);
+	RList *(*pids)(RDebug *dbg, int pid);
+	RList *(*tids)(RDebug *dbg, int pid);
+	RFList (*backtrace)(RDebug *dbg, int count);
 	/* flow */
 	int (*stop)(RDebug *dbg);
 	int (*step)(RDebug *dbg);
 	int (*step_over)(RDebug *dbg);
 	int (*cont)(RDebug *dbg, int pid, int tid, int sig);
 	int (*wait)(RDebug *dbg, int pid);
-	int (*kill)(RDebug *dbg, int pid, int tid, int sig);
+	bool (*gcore)(RDebug *dbg, RBuffer *dest);
+	bool (*kill)(RDebug *dbg, int pid, int tid, int sig);
 	RList* (*kill_list)(RDebug *dbg);
 	int (*contsc)(RDebug *dbg, int pid, int sc);
 	RList* (*frames)(RDebug *dbg, ut64 at);
@@ -254,7 +358,6 @@ typedef struct r_debug_plugin_t {
 	int (*drx)(RDebug *dbg, int n, ut64 addr, int size, int rwx, int g);
 	RDebugDescPlugin desc;
 	// TODO: use RList here
-	struct list_head list;
 } RDebugPlugin;
 
 // TODO: rename to r_debug_process_t ? maybe a thread too ?
@@ -262,56 +365,57 @@ typedef struct r_debug_pid_t {
 	int pid;
 	char status; /* stopped, running, zombie, sleeping ,... */
 	int runnable; /* when using 'run', 'continue', .. this proc will be runnable */
+	bool signalled;
 	char *path;
+	int uid;
+	int gid;
 	ut64 pc;
 } RDebugPid;
 
-enum RDebugReasonType {
-	R_DEBUG_REASON_DEAD = -1,
-	R_DEBUG_REASON_NONE = 0,
-	R_DEBUG_REASON_SIGNAL,
-	R_DEBUG_REASON_SEGFAULT,
-	R_DEBUG_REASON_BREAKPOINT,
-	R_DEBUG_REASON_READERR,
-	R_DEBUG_REASON_STEP,
-	R_DEBUG_REASON_ABORT,
-	R_DEBUG_REASON_WRITERR,
-	R_DEBUG_REASON_DIVBYZERO,
-	R_DEBUG_REASON_ILLEGAL,
-	R_DEBUG_REASON_UNKNOWN,
-	R_DEBUG_REASON_ERROR,
-	R_DEBUG_REASON_NEW_PID,
-	R_DEBUG_REASON_NEW_TID,
-	R_DEBUG_REASON_NEW_LIB,
-	R_DEBUG_REASON_EXIT_PID,
-	R_DEBUG_REASON_EXIT_TID,
-	R_DEBUG_REASON_EXIT_LIB,
-	R_DEBUG_REASON_TRAP,
-	R_DEBUG_REASON_SWI,
-	R_DEBUG_REASON_INT,
-	R_DEBUG_REASON_FPU,
-};
-
+/*
+ * Radare's debugger has both an external and internal API.
+ *
+ * TODO(jjd): reconcile external API and extend it for better funcitonality
+ * when using R2 as a library.
+ */
 #ifdef R_API
+R_API RDebug *r_debug_new(int hard);
+R_API RDebug *r_debug_free(RDebug *dbg);
+
 R_API int r_debug_attach(RDebug *dbg, int pid);
 R_API int r_debug_detach(RDebug *dbg, int pid);
 R_API int r_debug_startv(RDebug *dbg, int argc, char **argv);
 R_API int r_debug_start(RDebug *dbg, const char *cmd);
-R_API int r_debug_stop_reason(RDebug *dbg);
+
+/* reason we stopped */
+R_API RDebugReasonType r_debug_stop_reason(RDebug *dbg);
 R_API const char *r_debug_reason_to_string(int type);
-R_API int r_debug_wait(RDebug *dbg);
+
+/* wait for another event */
+R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp);
+
+/* continuations */
+R_API int r_debug_step(RDebug *dbg, int steps);
 R_API int r_debug_step_over(RDebug *dbg, int steps);
 R_API int r_debug_continue_until(RDebug *dbg, ut64 addr);
 R_API int r_debug_continue_until_optype(RDebug *dbg, int type, int over);
 R_API int r_debug_continue_until_nontraced(RDebug *dbg);
 R_API int r_debug_continue_syscall(RDebug *dbg, int sc);
 R_API int r_debug_continue_syscalls(RDebug *dbg, int *sc, int n_sc);
+R_API int r_debug_continue(RDebug *dbg);
+R_API int r_debug_continue_kill(RDebug *dbg, int signal);
+#if __WINDOWS__ && !__CYGWIN__
+R_API int r_debug_continue_pass_exception(RDebug *dbg);
+#endif
+
+/* process/thread handling */
+R_API int r_debug_select(RDebug *dbg, int pid, int tid);
 //R_API int r_debug_pid_add(RDebug *dbg);
 //R_API int r_debug_pid_add_thread(RDebug *dbg);
 //R_API int r_debug_pid_del(RDebug *dbg);
 //R_API int r_debug_pid_del_thread(RDebug *dbg);
 R_API int r_debug_pid_list(RDebug *dbg, int pid, char fmt);
-R_API RDebugPid *r_debug_pid_new(const char *path, int pid, char status, ut64 pc);
+R_API RDebugPid *r_debug_pid_new(const char *path, int pid, int uid, char status, ut64 pc);
 R_API RDebugPid *r_debug_pid_free(RDebugPid *pid);
 R_API RList *r_debug_pids(RDebug *dbg, int pid);
 
@@ -321,9 +425,6 @@ R_API bool r_debug_use(RDebug *dbg, const char *str);
 R_API RDebugInfo *r_debug_info(RDebug *dbg, const char *arg);
 R_API void r_debug_info_free (RDebugInfo *rdi);
 
-R_API RDebug *r_debug_new(int hard);
-R_API RDebug *r_debug_free(RDebug *dbg);
-
 /* send signals */
 R_API void r_debug_signal_init(RDebug *dbg);
 R_API int r_debug_signal_send(RDebug *dbg, int num);
@@ -337,15 +438,11 @@ R_API int r_debug_kill(RDebug *dbg, int pid, int tid, int sig);
 R_API RList *r_debug_kill_list(RDebug *dbg);
 // XXX: must be uint64 action
 R_API int r_debug_kill_setup(RDebug *dbg, int sig, int action);
-R_API int r_debug_step(RDebug *dbg, int steps);
-R_API int r_debug_continue(RDebug *dbg);
-R_API int r_debug_continue_kill(RDebug *dbg, int signal);
-R_API int r_debug_select(RDebug *dbg, int pid, int tid);
 
 /* handle.c */
 R_API void r_debug_plugin_init(RDebug *dbg);
 R_API int r_debug_plugin_set(RDebug *dbg, const char *str);
-R_API int r_debug_plugin_list(RDebug *dbg);
+R_API int r_debug_plugin_list(RDebug *dbg, int mode);
 R_API bool r_debug_plugin_add(RDebug *dbg, RDebugPlugin *foo);
 
 /* memory */
@@ -357,7 +454,7 @@ R_API RDebugMap *r_debug_map_get(RDebug *dbg, ut64 addr);
 R_API RDebugMap *r_debug_map_new (char *name, ut64 addr, ut64 addr_end, int perm, int user);
 R_API void r_debug_map_free(RDebugMap *map);
 R_API void r_debug_map_list(RDebug *dbg, ut64 addr, int rad);
-R_API void r_debug_map_list_visual(RDebug *dbg, ut64 addr, int use_color);
+R_API void r_debug_map_list_visual(RDebug *dbg, ut64 addr, int use_color, int cons_cols);
 
 /* descriptors */
 R_API RDebugDesc *r_debug_desc_new (int fd, char* path, int perm, int type, int off);
@@ -375,7 +472,7 @@ R_API int r_debug_reg_sync(RDebug *dbg, int type, int write);
 R_API int r_debug_reg_list(RDebug *dbg, int type, int size, int rad, const char *use_color);
 R_API int r_debug_reg_set(RDebug *dbg, const char *name, ut64 num);
 R_API ut64 r_debug_reg_get(RDebug *dbg, const char *name);
-R_API ut64 r_debug_reg_get_err(RDebug *dbg, const char *name, int *err);
+R_API ut64 r_debug_reg_get_err(RDebug *dbg, const char *name, int *err, utX *value);
 
 R_API void r_debug_io_bind(RDebug *dbg, RIO *io);
 R_API ut64 r_debug_execute(RDebug *dbg, const ut8 *buf, int len, int restore);
@@ -386,25 +483,28 @@ R_API int r_debug_stop(RDebug *dbg);
 /* backtrace */
 R_API RList *r_debug_frames(RDebug *dbg, ut64 at);
 
-R_API int r_debug_is_dead(RDebug *dbg);
+R_API bool r_debug_is_dead(RDebug *dbg);
 R_API int r_debug_map_protect(RDebug *dbg, ut64 addr, int size, int perms);
 /* args XXX: weird food */
 R_API ut64 r_debug_arg_get(RDebug *dbg, int fast, int num);
 R_API bool r_debug_arg_set(RDebug *dbg, int fast, int num, ut64 value);
 
+/* breakpoints (most in r_bp, this calls those) */
+R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, char *module, st64 m_delta);
+
 /* pid */
 R_API int r_debug_thread_list(RDebug *dbg, int pid);
 
 R_API void r_debug_tracenodes_reset(RDebug *dbg);
 
 R_API void r_debug_trace_reset(RDebug *dbg);
-R_API int r_debug_trace_pc(RDebug *dbg);
+R_API int r_debug_trace_pc(RDebug *dbg, ut64 pc);
 R_API void r_debug_trace_at(RDebug *dbg, const char *str);
 R_API RDebugTracepoint *r_debug_trace_get(RDebug *dbg, ut64 addr);
 R_API void r_debug_trace_list(RDebug *dbg, int mode);
 R_API RDebugTracepoint *r_debug_trace_add(RDebug *dbg, ut64 addr, int size);
 R_API RDebugTrace *r_debug_trace_new(void);
-R_API void r_debug_trace_free(RDebug *dbg);
+R_API void r_debug_trace_free(RDebugTrace *dbg);
 R_API int r_debug_trace_tag(RDebug *dbg, int tag);
 R_API int r_debug_child_fork(RDebug *dbg);
 R_API int r_debug_child_clone(RDebug *dbg);
@@ -425,14 +525,34 @@ R_API int r_debug_esil_watch_empty(RDebug *dbg);
 R_API void r_debug_esil_prestep (RDebug *d, int p);
 
 /* snap */
+R_API RDebugSnap* r_debug_snap_new(void);
 R_API void r_debug_snap_free(void *snap);
 R_API int r_debug_snap_delete(RDebug *dbg, int idx);
 R_API void r_debug_snap_list(RDebug *dbg, int idx, int mode);
-R_API int r_debug_snap_diff(RDebug *dbg, int idx);
 R_API int r_debug_snap(RDebug *dbg, ut64 addr);
 R_API int r_debug_snap_comment (RDebug *dbg, int idx, const char *msg);
+R_API RDebugSnapDiff* r_debug_snap_map(RDebug *dbg, RDebugMap *map);
 R_API int r_debug_snap_all(RDebug *dbg, int perms);
 R_API RDebugSnap* r_debug_snap_get (RDebug *dbg, ut64 addr);
+R_API int r_debug_snap_set_idx (RDebug *dbg, int idx);
+R_API int r_debug_snap_set (RDebug *dbg, RDebugSnap *snap);
+
+/* snap diff */
+R_API void r_debug_diff_free (void *p);
+R_API RDebugSnapDiff* r_debug_diff_add(RDebug *dbg, RDebugSnap *base);
+R_API void r_debug_diff_set(RDebug *dbg, RDebugSnapDiff *diff);
+
+/* page data */
+R_API void r_page_data_free(void *p);
+
+/* debug session */
+R_API void r_debug_session_free (void *p) ;
+R_API void r_debug_session_list (RDebug *dbg);
+R_API bool r_debug_session_add (RDebug *dbg);
+R_API void r_debug_session_set (RDebug *dbg, RDebugSession *session);
+R_API bool r_debug_session_set_idx (RDebug *dbg, int idx);
+R_API RDebugSession* r_debug_session_get (RDebug *dbg, ut64 addr);
+R_API int r_debug_step_back (RDebug *dbg);
 
 /* plugin pointers */
 extern RDebugPlugin r_debug_plugin_native;
@@ -440,8 +560,10 @@ extern RDebugPlugin r_debug_plugin_esil;
 extern RDebugPlugin r_debug_plugin_rap;
 extern RDebugPlugin r_debug_plugin_gdb;
 extern RDebugPlugin r_debug_plugin_bf;
+extern RDebugPlugin r_debug_plugin_io;
 extern RDebugPlugin r_debug_plugin_wind;
-
+extern RDebugPlugin r_debug_plugin_bochs;
+extern RDebugPlugin r_debug_plugin_qnx;
 #endif
 
 #ifdef __cplusplus
