@@ -1,6 +1,11 @@
 #ifndef R2_SOCKET_H
 #define R2_SOCKET_H
 
+/* Must be included before windows.h (r_types) */
+#if defined(__WINDOWS__) && !defined(__CYGWIN__) && !defined(MINGW32) && !defined(__MINGW64__)
+#include <ws2tcpip.h>
+#endif
+
 #include "r_types.h"
 
 #ifdef __cplusplus
@@ -24,8 +29,8 @@ R_LIB_VERSION_HEADER(r_socket);
 #include <openssl/err.h>
 #endif
 
-#if defined(__WINDOWS__) && !defined(__CYGWIN__) && !defined(MINGW32) && !defined(__MINGW64__)
-#include <ws2tcpip.h>
+#if __UNIX__ || defined(__CYGWIN__)
+#include <netinet/tcp.h>
 #endif
 
 /* For the Mingw-W64 toolchain */
@@ -49,9 +54,12 @@ typedef struct {
 #endif
 } R2Pipe;
 
-
 typedef struct r_socket_t {
+#ifdef _MSC_VER
+	SOCKET fd;
+#else
 	int fd;
+#endif
 	int is_ssl;
 	int local; // TODO: merge ssl with local -> flags/options
 	int port;
@@ -205,16 +213,16 @@ typedef struct r_run_profile_t {
 	char *_input;
 	char *_connect;
 	char *_listen;
+	int _pty;
 	int _timeout;
 	int _timeout_sig;
 	int _nice;
 } RRunProfile;
 
 R_API RRunProfile *r_run_new(const char *str);
-R_API int r_run_parseline (RRunProfile *p, char *b);
-R_API int r_run_parse(RRunProfile *pf, const char *profile);
+R_API bool r_run_parse(RRunProfile *pf, const char *profile);
 R_API void r_run_free (RRunProfile *r);
-R_API int r_run_parseline (RRunProfile *p, char *b);
+R_API bool r_run_parseline (RRunProfile *p, char *b);
 R_API const char *r_run_help(void);
 R_API int r_run_config_env(RRunProfile *p);
 R_API int r_run_start(RRunProfile *p);
