@@ -5,7 +5,7 @@
 
 #define DEBUGGER 1
 
-#if __WIN32__ || __CYGWIN__ || MINGW32
+#if __WINDOWS__ || __CYGWIN__ || MINGW32
 #define R2_PREFIX "."
 #define R2_LIBDIR "./lib"
 #define R2_INCDIR "./include/libr"
@@ -17,7 +17,7 @@
 #define R2_DATDIR "/usr/share"
 #endif
 
-#define R2_VERSION "1.0.2"
+#define R2_VERSION "1.5.0"
 #define HAVE_LIB_MAGIC 1
 #define USE_LIB_MAGIC 0
 #ifndef HAVE_LIB_SSL
